package xyz.ottr.spec.wottr;

import java.io.*;
import java.nio.file.*;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.jena.rdf.model.*;
import org.apache.jena.util.FileUtils;
import org.topbraid.shacl.validation.ValidationUtil;
import org.topbraid.shacl.vocabulary.SH;

public abstract class Util {

	final static String ROOT = Paths.get(System.getProperty("user.dir")).getParent().toString() + "/";
	
	final static String GRAMMAR = "core-grammar.shacl.ttl";
	final static String TYPES = "types.shacl.ttl";
	final static String VOCABULARY = "core-vocabulary.owl.ttl";
	final static String TOKENS = "ottr-tokens.shacl.ttl";
	final static String STD_TOKENS = "std-tokens.shacl.ttl";
	
	//// IO

	static String rootPath(String path) {
		return path.replaceFirst(ROOT, "");
	}

	static List<String> getFiles (String folder) throws IOException {
		return Files.list(Paths.get(Util.ROOT + "/" + folder))
				.filter(Files::isRegularFile)
				.map(path -> rootPath(path.toString()))
				.sorted()
				.collect(Collectors.toList());
	}

	//// Jena utils

	static Model getModel (String... urls) {
		Model model = ModelFactory.createDefaultModel();
		if (urls != null) {
			for (String url : urls) {
				url = url.startsWith("http") ? url : ROOT + url;
				model.read(url, FileUtils.guessLang(url, FileUtils.langTurtle));
			}
		}
		return model;
	}
	static void writeModel (Model model, String filename) throws FileNotFoundException {
		model.write(new FileOutputStream(filename), FileUtils.langTurtle);
	}
	static void printModel (Model model) {
		model.write(System.out, FileUtils.langTurtle);
	}

	//// Shacl utils

	static Resource getReport (Model shape, Model data) {
		return ValidationUtil.validateModel(data, shape, true);
	}
	static boolean isConforms (Resource report) {
		return report.getProperty(SH.conforms).getBoolean();
	}
}

